package com.redxun.knowledge.album.controller;

import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.knowledge.album.entity.consts.AlbumConst;
import com.redxun.knowledge.album.entity.dao.AlbumInfo;
import com.redxun.knowledge.album.entity.dto.*;
import com.redxun.knowledge.album.entity.vo.*;
import com.redxun.knowledge.album.service.*;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/album")
@Api(tags = "业务--专辑基本信息表")
@ClassDefine(title = "业务--专辑基本信息表", alias = "AlbumInfoController", path = "/api-knowledge/album", packages = "album", packageName = "子系统名称")
public class AlbumInfoController extends BaseController<AlbumInfo> {

    @Autowired
    AlbumInfoServiceImpl albumInfoService;

    @Autowired
    private AlbumApprovalNodeServiceImpl albumApprovalNodeService;

    @Autowired
    private AlbumApproveServiceImpl albumApprovalService;

    @Autowired
    private AlbumVersionServiceImpl albumVersionService;

    @Autowired
    private AlbumCategoryServiceImpl albumCategoryService;

    @Autowired
    private AlbumYelpServiceImpl albumYelpService;


    @Override
    public BaseService getBaseService() {
        return albumInfoService;
    }

    @Override
    public String getComment() {
        return "业务--专辑基本信息表";
    }

    @ApiOperation("PC端新增知识专辑")
    @PostMapping("create")
    public JsonResult create(@RequestBody AlbumInfoDto albumInfoDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        String pkId = albumInfoService.create(albumInfoDto);
        return jsonResult.setData(pkId);
    }

    @ApiOperation("PC端编辑知识专辑")
    @PostMapping("edit")
    public JsonResult edit(@RequestBody AlbumInfoEditDto albumInfoEditDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        String message = albumInfoService.edit(albumInfoEditDto);
        jsonResult.setMessage(message);
        return jsonResult.setData(albumInfoEditDto.getPkId());
    }

    @ApiOperation("根据知识专辑ID获取知识专辑详情数据")
    @GetMapping("detail")
    public JsonResult detail(@RequestParam("pkId") String pkId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        AlbumInfoDetailVo albumInfoDetailVo = albumInfoService.detail(pkId);
        return jsonResult.setData(albumInfoDetailVo);
    }

    @ApiOperation("根据知识专辑版本ID获取知识专辑版本的详情数据")
    @GetMapping("version")
    public JsonResult version(@RequestParam("pkId") String pkId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        AlbumVersionDetailVo albumVersionDetailVo = albumInfoService.version(pkId);
        return jsonResult.setData(albumVersionDetailVo);
    }

    @ApiOperation("删除知识专辑")
    @Override
    public JsonResult del(String ids) {
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        String del = albumInfoService.del(ids);
        jsonResult.setMessage(del);
        return jsonResult;
    }

    @ApiOperation("转移专辑负责人")
    @PostMapping("transferOwner")
    public JsonResult transferOwner(@RequestBody TransferOwnerDto transferOwnerDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        albumInfoService.transferOwner(transferOwnerDto);
        jsonResult.setMessage(AlbumConst.OPERATOR_SUCCESS);
        jsonResult.setData(transferOwnerDto.getPkId());
        return jsonResult;
    }

    @ApiOperation("关闭专辑")
    @PostMapping("close")
    public JsonResult close(@RequestBody TransferOwnerDto transferOwnerDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        albumInfoService.close(transferOwnerDto);
        return jsonResult.setData(transferOwnerDto.getPkId());
    }

    @ApiOperation("移除目录知识和删除点评和回复")
    @PostMapping("removeKnowledgeAndYelp")
    public JsonResult removeKnowledgeAndYelp(@RequestBody RemoveKnowledgeAndYelpDto removeKnowledgeAndYelpDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        albumInfoService.removeKnowledgeAndYelp(removeKnowledgeAndYelpDto);
        return jsonResult.setMessage(AlbumConst.OPERATOR_SUCCESS);
    }

    @ApiOperation("批量更新审核流程节点数据")
    @PostMapping("batchUpdateApproveFlows")
    public JsonResult batchUpdateApproveFlows(@RequestBody List<AlbumApprovalNodeDto> albumApprovalNodeDtoList){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        albumInfoService.batchUpdateApproveFlows(albumApprovalNodeDtoList);
        return jsonResult.setMessage(AlbumConst.BATCH_APPROVAL_SUCCESS);
    }
    
    @ApiOperation("获取当前审核流程列表数据")
    @GetMapping("approveFlows")
    public JsonResult approveFlows(@RequestParam("type") Integer type){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        if (type == 1){
            List<AlbumApproverNodeVo> albumApproverNodeVos = albumInfoService.approveFlowsAlbumApprovalNode();
            return jsonResult.setData(albumApproverNodeVos);
        }
        else if (type == 2){
            List<AlbumApproverNodeVo> albumApproverNodeVos = albumInfoService.approveFlowsAlbumYelpApprovalNode();
            return jsonResult.setData(albumApproverNodeVos);
        } else {
            return jsonResult.setMessage(AlbumConst.NO_TYPE);
        }
    }

    @ApiOperation("获取专辑审核流程模板")
    @GetMapping("approvalTemplate")
    public JsonResult approvalTemplate(){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<Map<String,Object>> result = albumApprovalNodeService.approvalTemplate();
        return jsonResult.setData(result);
    }

    @ApiOperation("对知识专辑审核中的版本进行审核")
    @PostMapping("approve")
    public JsonResult approve(@RequestBody AlbumApprovalRemarkDto albumApprovalRemarkDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        Integer result = albumApprovalService.approve(albumApprovalRemarkDto);
        if (result == 1){
            return jsonResult.setMessage(AlbumConst.APPROVAL_FAIL);
        } else if (result == 2){
            return jsonResult.setMessage(AlbumConst.APPROVAL_FAIL_AUTHORITY);
        } else {
            return jsonResult.setMessage(AlbumConst.APPROVAL_SUCCESS);
        }
    }

    @ApiOperation("PC端根据知识专辑ID获取审批流程列表数据")
    @GetMapping("approvals")
    public JsonResult approvals(@RequestParam("albumId") String albumId,
                                @RequestParam(value = "versionId",required = false) String versionId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        AlbumApproveFlowVo albumApproveFlowVoList = albumApprovalService.approvals(albumId,versionId);
        return jsonResult.setData(albumApproveFlowVoList);
    }

    @ApiOperation("PC端根据知识专辑ID获取知识专辑详情")
    @GetMapping("info")
    public JsonResult info(@RequestParam("pkId") String pkId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        AlbumInfoDetailPcVo albumInfoDetailPcVo = albumInfoService.info(pkId);
        return jsonResult.setData(albumInfoDetailPcVo);
    }

    @ApiOperation("PC端根据知识专辑ID和版本ID获取知识专辑目录及目录下知识")
    @GetMapping("catalogue")
    public JsonResult catalogue(@RequestParam("albumId") String albumId,@RequestParam(value = "versionId",required = false) String versionId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<AlbumCataloguePcVo> albumCatalogueVoList = albumVersionService.catalogue(albumId,versionId);
        return jsonResult.setData(albumCatalogueVoList);
    }

    @ApiOperation("PC端查询知识专辑分类列表数据")
    @GetMapping("categories")
    public JsonResult categories(@RequestParam(value = "pkId",required = false,defaultValue = "0")String pkId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<AlbumCategoryPcVo> albumCategoryPcVos = albumCategoryService.categories(pkId);
        return jsonResult.setData(albumCategoryPcVos);
    }

    @ApiOperation("PC端对专辑进行点评或者回复点评（回复）内容")
    @PostMapping("yelp")
    public JsonResult yelp(@RequestBody YelpDto yelpDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        String result = albumYelpService.yelp(yelpDto);
        return jsonResult.setMessage(result);
    }

}

