package com.redxun.knowledge.album.service;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.IdGenerator;
import com.redxun.common.tool.StringUtils;
import com.redxun.dto.user.OsUserDto;
import com.redxun.knowledge.album.entity.consts.AlbumConst;
import com.redxun.knowledge.album.entity.dao.*;
import com.redxun.knowledge.album.entity.dto.*;
import com.redxun.knowledge.album.entity.enums.AlbumInfoEnum;
import com.redxun.knowledge.album.entity.enums.AlbumVersionEnum;
import com.redxun.knowledge.album.entity.vo.*;
import com.redxun.knowledge.album.mapper.*;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.ActionLog;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.vo.KnowledgeCategoryAdminVo;
import com.redxun.knowledge.mapper.ActionLogMapper;
import com.redxun.knowledge.service.KnowledgeCategoryServiceImpl;
import com.redxun.knowledge.service.KnowledgeServiceImpl;
import com.redxun.knowledge.utils.PageListUtils;
import com.sun.xml.bind.v2.TODO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * [业务--专辑基本信息表]业务服务类
 */
@Slf4j
@Service
public class AlbumInfoServiceImpl extends SuperServiceImpl<AlbumInfoMapper, AlbumInfo> implements BaseService<AlbumInfo> {

    @Resource
    private AlbumInfoMapper albumInfoMapper;

    @Autowired
    private AlbumVersionMapper albumVersionMapper;

    @Autowired
    private AlbumCatalogueMapper albumCatalogueMapper;

    @Autowired
    private AlbumCatalogueKnowledgeMapper albumCatalogueKnowledgeMapper;

    @Autowired
    private AlbumCategoryMapper albumCategoryMapper;

    @Autowired
    private AlbumYelpMapper albumYelpMapper;

    @Autowired
    private AlbumApprovalNodeServiceImpl albumApprovalNodeService;

    @Autowired
    private AlbumYelpApprovalNodeServiceImpl albumYelpApprovalNodeService;

    @Autowired
    private AlbumApproveServiceImpl albumApproveService;

    @Autowired
    private AlbumCatalogueServiceImpl albumCatalogueService;

    @Autowired
    private AlbumYelpServiceImpl albumYelpService;

    @Autowired
    private AlbumYelpApproveServiceImpl albumYelpApproveService;

    @Autowired
    private ActionLogMapper actionLogMapper;

    @Autowired
    private UserService userService;

    @Autowired
    private ThreadPoolExecutor threadPoolExecutor;

    @Override
    public BaseDao<AlbumInfo> getRepository() {
        return albumInfoMapper;
    }

    /**
     * 新增专辑
     *
     * @param albumInfoDto 专辑基本信息&专辑目录&目录下收录知识
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String create(AlbumInfoDto albumInfoDto) {
        AlbumInfo albumInfo = new AlbumInfo();
        BeanUtils.copyProperties(albumInfoDto, albumInfo);
        albumInfo.setPkId(IdGenerator.getIdStr());
        String userId = userService.queryLoginUser();
        // 获取当前登录人用户信息
        OsUserDto osUserDto = userService.queryOsUserDto(userId);
        // 加入用户信息数据
        albumInfo.setOwnerCompanyId(osUserDto.getCompanyId());
        albumInfo.setOwnerDepartmentId(osUserDto.getDeptId());
        albumInfo.setOwnerId(osUserDto.getUserId());
        albumInfo.setUpdateBy(osUserDto.getUserId());
        //新增专辑
        albumInfoMapper.insert(albumInfo);
        //生成一号版本
        AlbumVersion albumVersion = new AlbumVersion();
        albumVersion.setPkId(IdGenerator.getIdStr());
        albumVersion.setAlbumId(albumInfo.getPkId());
        albumVersion.setApplyUserId(osUserDto.getUserId());
        albumVersion.setSummary(albumInfoDto.getSummary());
        albumVersion.setUpdateBy(osUserDto.getUserId());
        albumVersionMapper.insert(albumVersion);
        //生成目录
        if (CollectionUtils.isNotEmpty(albumInfoDto.getCatalogues())) {
            createCatalogue(albumInfoDto.getCatalogues(), albumInfo.getPkId(), albumVersion.getPkId());
        }
        //生成最新版审核流程数据
        createAlbumApprove(albumInfo.getPkId(), albumVersion.getPkId());

        return albumInfo.getPkId();
    }

    public IPage query(QueryFilter queryFilter) {
        Map<String, Object> params = PageHelper.constructParams(queryFilter);
        String userId = userService.queryLoginUser();
        //构建PC端参数
        Map<String, Object> map = new HashMap<>();
        //PC端条件过滤
        if (queryFilter.getParams().containsKey("scope")) {
            String scope = (String) queryFilter.getParams().get("scope");

            switch (scope) {
                case "1":
                    //我的专辑
                    queryFilter.getParams().put("ownerIds", userService.queryLoginUser());
                    //查询我的专辑
                    IPage page1 = albumInfoMapper.findAllAlbumInfoByOwnerIds(queryFilter.getPage(), queryFilter.getParams(), params);
                    List<AlbumInfo> records = page1.getRecords();
                    List<AlbumInfoVo> albumInfoVoList = pageInfo(records);
                    page1.setRecords(albumInfoVoList);
                    return page1;
                case "2":
                    //我的专辑
                    queryFilter.getParams().put("ownerId", userService.queryLoginUser());
                    List<AlbumInfo> ownerIdsList = albumInfoMapper.findAllAlbumInfoByOwnerIdsNoPage(queryFilter.getParams(), params);
                    //已发布的专辑
                    queryFilter.getParams().put("versionState", 2);
                    List<AlbumInfo> albumInfoList = albumInfoMapper.findAllAlbumInfoByVersionState(queryFilter.getParams(), params);
                    if (albumInfoList != null) {
                        ownerIdsList.addAll(albumInfoList);
                    }
                    List<AlbumInfo> collect = ownerIdsList.stream().distinct().collect(Collectors.toList());
                    List<AlbumInfoVo> albumInfoVoListResult = pageInfo(collect);
                    //不是我创建的专辑(状态设置为2 已发布)
                    albumInfoVoListResult.stream().filter(e -> !e.getOwnerId().equals(userId)).forEach(h -> {
                        h.setVersionState(2);
                        h.setVersionStateName(AlbumVersionEnum.getMessage(2));
                    });
                    Page page2 = PageListUtils.getPages(queryFilter.getPage().getCurrent(), queryFilter.getPage().getSize(), albumInfoVoListResult);
                    return page2;
                case "3":
                    //查询我点评过得所有专辑Id
                    List<AlbumYelp> albumYelpList = albumYelpService.findAll(new LambdaQueryWrapper<AlbumYelp>().
                            eq(AlbumYelp::getCreateBy, userId));
                    List<AlbumInfo> albumInfos = albumYelpList.parallelStream().map(albumYelp -> {
                        AlbumInfo albumInfo = albumInfoMapper.selectById(albumYelp.getAlbumId());
                        return albumInfo;
                    }).collect(Collectors.toList());
                    List<AlbumInfoVo> albumInfoVoListYelp = pageInfo(albumInfos.stream().filter(e -> e.getState() == 1).distinct().
                            sorted((t1, t2) -> t2.getCreateTime().compareTo(t1.getCreateTime())).collect(Collectors.toList()));
                    Page page3 = PageListUtils.getPages(queryFilter.getPage().getCurrent(), queryFilter.getPage().getSize(), albumInfoVoListYelp);
                    return page3;
            }
        }
        IPage page = albumInfoMapper.findAllAlbumInfo(queryFilter.getPage(), queryFilter.getParams(), params);
        List<AlbumInfo> collect = page.getRecords();
        List<AlbumInfoVo> albumInfoVoList = pageInfo(collect);
        page.setRecords(albumInfoVoList);
        return page;
    }

    /**
     * 分页方法封装
     *
     * @param collect
     * @return
     */
    private List<AlbumInfoVo> pageInfo(List<AlbumInfo> collect) {
        String userId = userService.queryLoginUser();
        List<AlbumInfoVo> albumInfoVoList = collect.parallelStream().map(e -> {
            AlbumInfoVo albumInfoVo = new AlbumInfoVo();
            BeanUtils.copyProperties(e, albumInfoVo);
            //专辑分类信息
            CompletableFuture<Void> categoryCompletableFuture = CompletableFuture.runAsync(() -> {
                String categoryId = e.getCategoryId();
                AlbumCategory albumCategory = albumCategoryMapper.selectById(categoryId);
                if (albumCategory.getGrade() == 1) {
                    albumInfoVo.setCategoryName(albumCategory.getName());
                } else {
                    AlbumCategory albumCategoryParent = albumCategoryMapper.
                            selectOne(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getPkId, albumCategory.getParent()));
                    albumInfoVo.setCategoryName(albumCategoryParent.getName() + "|" + albumCategory.getName());
                }
            }, threadPoolExecutor);
            //专辑收录知识量(当前生效版本各个目录中知识数量总和)
            CompletableFuture<Void> countKnowledgeCompletableFuture = CompletableFuture.runAsync(() -> {
                AlbumVersion albumVersion = albumVersionMapper.
                        selectOne(new LambdaQueryWrapper<AlbumVersion>().
                                eq(AlbumVersion::getAlbumId, e.getPkId()).
                                eq(AlbumVersion::getState, 2));
                //没有生效的专辑(第一次创建专辑时的情况)
                if (albumVersion == null) {
                    AlbumVersion albumVersionFirst = albumVersionMapper.
                            selectOne(new LambdaQueryWrapper<AlbumVersion>().
                                    eq(AlbumVersion::getAlbumId, e.getPkId()).
                                    eq(AlbumVersion::getState, 1));
                    if (albumVersionFirst == null) {
                        AlbumVersion albumVersionFour = albumVersionMapper.
                                selectOne(new LambdaQueryWrapper<AlbumVersion>().
                                        eq(AlbumVersion::getAlbumId, e.getPkId()).
                                        eq(AlbumVersion::getState, 4));
                        Integer countKnowledge = albumInfoMapper.selectCountKnowledge(e.getPkId(), albumVersionFour.getPkId());
                        albumInfoVo.setKnowledgeTotal(countKnowledge);
                    } else {
                        Integer countKnowledge = albumInfoMapper.selectCountKnowledge(e.getPkId(), albumVersionFirst.getPkId());
                        albumInfoVo.setKnowledgeTotal(countKnowledge);
                    }
                } else {
                    Integer countKnowledge = albumInfoMapper.selectCountKnowledge(e.getPkId(), albumVersion.getPkId());
                    albumInfoVo.setKnowledgeTotal(countKnowledge);
                }
            }, threadPoolExecutor);
            //专辑点评量(当前专辑中通过审核的点评量总和)
            CompletableFuture<Void> countYelpCompletableFuture = CompletableFuture.runAsync(() -> {
                Integer countYelp = albumYelpMapper.selectCount(new LambdaQueryWrapper<AlbumYelp>().
                        eq(AlbumYelp::getAlbumId, e.getPkId()).
                        eq(AlbumYelp::getState, 1));
                albumInfoVo.setYelpTotal(countYelp);
            }, threadPoolExecutor);
            //其他参数
            CompletableFuture<Void> anotherCompletableFuture = CompletableFuture.runAsync(() -> {
                //专辑负责人姓名
                albumInfoVo.setOwnerName(userService.queryOsUserDto(e.getOwnerId()).getFullName());
                //专辑负责人头像
                albumInfoVo.setOwnerPhoto((String) userService.querySexAndPhoto(e.getOwnerId()).get("photo"));
                //专辑状态名称(0 显示专辑状态,不显示版本状态 1 显示版本状态)
                List<AlbumVersion> albumVersions = albumVersionMapper.selectList(new LambdaQueryWrapper<AlbumVersion>().
                        eq(AlbumVersion::getAlbumId, e.getPkId()).orderByDesc(AlbumVersion::getCreateTime));
                if (e.getState() == 0) {
                    albumInfoVo.setStateName(AlbumInfoEnum.getMessage(e.getState()));
                } else {
                    albumInfoVo.setVersionState(albumVersions.get(0).getState());
                    albumInfoVo.setVersionStateName(AlbumVersionEnum.getMessage(albumVersions.get(0).getState()));
                }
                //操作人姓名及时间
                albumInfoVo.setOperator(userService.queryOsUserDto(e.getUpdateBy()).getFullName());
                albumInfoVo.setUpdateTime(e.getUpdateTime());
                //动作标记(M 管理端)
                List<String> actionFlagM = new ArrayList<>();
                if (e.getState() == 0) {
                    actionFlagM.add("edit");
                    actionFlagM.add("delete");
                } else if (e.getState() == 1 && albumVersions.get(0).getState() == 1) {
                    actionFlagM.add("view");
                } else if (e.getState() == 1 && albumVersions.get(0).getState() == 2) {
                    actionFlagM.add("edit");
                } else if (e.getState() == 1 && albumVersions.get(0).getState() == 4) {
                    actionFlagM.add("view");
                }
                //动作标记PC端
                List<String> actionFlag = new ArrayList<>();
                if (albumVersions.get(0).getState() == 2 && e.getOwnerId().equals(userId)) {
                    actionFlag.add("edit");
                } else if (albumVersions.get(0).getState() == 4 && e.getOwnerId().equals(userId)) {
                    actionFlag.add("delete");
                    actionFlag.add("edit");
                }
                albumInfoVo.setActionFlag(actionFlag);
                albumInfoVo.setActionFlagM(actionFlagM);
            }, threadPoolExecutor);
            CompletableFuture.allOf(
                    categoryCompletableFuture,
                    countKnowledgeCompletableFuture,
                    countYelpCompletableFuture,
                    anotherCompletableFuture).join();
            return albumInfoVo;
        }).collect(Collectors.toList());
        return albumInfoVoList;
    }

    /**
     * 专辑详情
     *
     * @param pkId
     * @return
     */
    public AlbumInfoDetailVo detail(String pkId) {
        AlbumInfoDetailVo albumInfoDetailVo = new AlbumInfoDetailVo();
        AlbumInfo albumInfo = albumInfoMapper.selectById(pkId);
        BeanUtils.copyProperties(albumInfo, albumInfoDetailVo);
        //专辑负责人姓名
        albumInfoDetailVo.setOwner(userService.queryOsUserDto(albumInfo.getOwnerId()).getFullName());
        //所属公司名称
        albumInfoDetailVo.setOwnerCompany(userService.findDeptByDeptId(albumInfo.getOwnerCompanyId()).getName());
        //所属部门名称
        albumInfoDetailVo.setOwnerDepartment(userService.findDeptByDeptId(albumInfo.getOwnerDepartmentId()).getName());
        //分类名称
        AlbumCategory albumCategory = albumCategoryMapper.selectById(albumInfo.getCategoryId());
        if (albumCategory.getGrade() == 1) {
            albumInfoDetailVo.setCategoryName(albumCategory.getName());
        } else {
            AlbumCategory albumCategoryParent = albumCategoryMapper.
                    selectOne(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getPkId, albumCategory.getParent()));
            albumInfoDetailVo.setCategoryName(albumCategoryParent.getName() + "|" + albumCategory.getName());
        }
        //状态名称
        albumInfoDetailVo.setStateName(AlbumInfoEnum.getMessage(albumInfo.getState()));
        //专辑点评量(当前专辑中通过审核的点评量总和)
        Integer countYelp = albumYelpMapper.selectCount(new LambdaQueryWrapper<AlbumYelp>().
                eq(AlbumYelp::getAlbumId, pkId).
                eq(AlbumYelp::getState, 1));
        albumInfoDetailVo.setYelpTotal(countYelp);
        //专辑版本信息
        QueryWrapper<AlbumVersion> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("ALBUM_ID", pkId);
        queryWrapper.orderByDesc("CREATE_TIME_");
        List<AlbumVersion> albumVersions = albumVersionMapper.selectList(queryWrapper);
        //最新版本状态
        albumInfoDetailVo.setVersionState(albumVersions.get(0).getState());
        if (albumVersions.get(0).getState() == 3){
            albumInfoDetailVo.setVersionStateName("");
        } else {
            albumInfoDetailVo.setVersionStateName(AlbumVersionEnum.getMessage(albumVersions.get(0).getState()));
        }
        //专辑版本集合
        List<AlbumVersionVo> albumVersionVoList = albumVersions.parallelStream().map(albumVersion -> {
            AlbumVersionVo albumVersionVo = new AlbumVersionVo();
            BeanUtils.copyProperties(albumVersion, albumVersionVo);
            //版本状态名称
            if (albumVersion.getState() == 3){
                albumVersionVo.setStateName("");
            } else {
                albumVersionVo.setStateName(AlbumVersionEnum.getMessage(albumVersion.getState()));
            }
            return albumVersionVo;
        }).collect(Collectors.toList());
        albumInfoDetailVo.setVersions(albumVersionVoList);
        //专辑详情动作标记
        //审核中版本数量
        long count1 = albumVersions.stream().filter(e -> e.getState() == 1).count();
        //生效中版本数量
        long count2 = albumVersions.stream().filter(e -> e.getState() == 2).count();
        List<String> actionFlag = new ArrayList<>();
        //专辑负责人
        if (albumInfo.getState() == 1 && count2 > 0 && count1 == 0) {
            actionFlag.add("edit");
            actionFlag.add("transfer");
            actionFlag.add("close");
        } else if (albumInfo.getState() == 0 && count2 > 0 && count1 == 0) {
            actionFlag.add("edit");
            actionFlag.add("transfer");
            actionFlag.add("open");
        }
        albumInfoDetailVo.setActionFlag(actionFlag);
        return albumInfoDetailVo;
    }

    /**
     * 专辑版本详情
     *
     * @param pkId
     * @return
     */
    public AlbumVersionDetailVo version(String pkId) {
        AlbumVersionDetailVo albumVersionDetailVo = new AlbumVersionDetailVo();
        AlbumVersion albumVersion = albumVersionMapper.selectById(pkId);
        albumVersionDetailVo.setPkId(albumVersion.getPkId());
        albumVersionDetailVo.setSummary(albumVersion.getSummary());
        //查询审核Id
        List<AlbumApprove> albumApproveLists = albumApproveService.
                findAll(new LambdaQueryWrapper<AlbumApprove>().
                        eq(AlbumApprove::getVersionId, pkId).
                        isNull(AlbumApprove::getResult).
                        orderByAsc(AlbumApprove::getSort));
        if (CollectionUtils.isNotEmpty(albumApproveLists)) {
            albumVersionDetailVo.setApprovalId(albumApproveLists.get(0).getPkId());
        }
        //目录下知识的动作标记
        List<String> actionFlag = new ArrayList<>();
        if (albumVersion.getState() == 1) {
            actionFlag.add("view");
        } else if (albumVersion.getState() == 2) {
            actionFlag.add("delete");
            actionFlag.add("view");
        } else if (albumVersion.getState() == 3) {
            actionFlag.add("view");
        }
        //审批流程节点
        QueryWrapper<AlbumApprove> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("ALBUM_ID", albumVersion.getAlbumId());
        queryWrapper.eq("VERSION_ID", pkId);
        queryWrapper.orderByAsc("SORT");
        List<AlbumApprove> albumApproveList = albumApproveService.findAll(queryWrapper);
        List<AlbumApproveVo> albumApproveVoList = new ArrayList<>();
        boolean flag = true;
        for (int i = 0; i < albumApproveList.size(); i++) {
            AlbumApproveVo albumApproveVo = new AlbumApproveVo();
            BeanUtils.copyProperties(albumApproveList.get(i), albumApproveVo);
            albumApproveVo.setComment(albumApproveList.get(i).getRemark());
            if (albumApproveList.get(i).getResult() == null) {
                if (flag) {
                    albumApproveVo.setApprovalStatus("提交审核");
                    flag = false;
                } else {
                    albumApproveVo.setApprovalStatus("");
                }
            } else if (albumApproveList.get(i).getResult() == 0) {
                albumApproveVo.setApprovalStatus("审核驳回");
                flag = false;
            } else if (albumApproveList.get(i).getResult() == 1) {
                albumApproveVo.setApprovalStatus("审核通过");
            }
            albumApproveVoList.add(albumApproveVo);
        }
        albumVersionDetailVo.setApprovalFlows(albumApproveVoList);
        return albumVersionDetailVo;
    }

    /**
     * 批量更新审核流程节点数据
     *
     * @param albumApprovalNodeDtoList
     */
    @Transactional(rollbackFor = Exception.class)
    public void batchUpdateApproveFlows(List<AlbumApprovalNodeDto> albumApprovalNodeDtoList) {
        //专辑审核节点插入
        List<AlbumApprovalNodeDto> albumApprovalNodeDto1 = albumApprovalNodeDtoList.
                stream().filter(e -> e.getType() == 1).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(albumApprovalNodeDto1)) {
            //清除原有数据
            albumApprovalNodeService.deleteAll();
            //批量插入数据
            List<AlbumApprovalNode> albumApprovalNodeList = albumApprovalNodeDto1.parallelStream().map(e -> {
                AlbumApprovalNode albumApprovalNode = new AlbumApprovalNode();
                albumApprovalNode.setPkId(IdGenerator.getIdStr());
                BeanUtils.copyProperties(e, albumApprovalNode);
                return albumApprovalNode;

            }).collect(Collectors.toList());
            albumApprovalNodeService.saveBatch(albumApprovalNodeList);
        }

        //点评审核节点插入
        List<AlbumApprovalNodeDto> albumApprovalNodeDto2 = albumApprovalNodeDtoList.
                stream().filter(e -> e.getType() == 2).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(albumApprovalNodeDto2)) {
            //清除原有数据
            albumYelpApprovalNodeService.deleteAll();
            //批量插入数据
            List<AlbumYelpApprovalNode> albumYelpApprovalNodeList = albumApprovalNodeDto2.parallelStream().map(e -> {
                AlbumYelpApprovalNode albumYelpApprovalNode = new AlbumYelpApprovalNode();
                albumYelpApprovalNode.setPkId(IdGenerator.getIdStr());
                BeanUtils.copyProperties(e, albumYelpApprovalNode);
                return albumYelpApprovalNode;
            }).collect(Collectors.toList());
            albumYelpApprovalNodeService.saveBatch(albumYelpApprovalNodeList);
        }
    }

    /**
     * 查询专辑审核节点表数据
     *
     * @return
     */
    public List<AlbumApproverNodeVo> approveFlowsAlbumApprovalNode() {

        List<AlbumApprovalNode> albumApprovalNodeList = albumApprovalNodeService.findAll(null);
        List<AlbumApproverNodeVo> collect = albumApprovalNodeList.parallelStream().map(e -> {
            AlbumApproverNodeVo albumApproverNodeVo = new AlbumApproverNodeVo();
            BeanUtils.copyProperties(e, albumApproverNodeVo);
            return albumApproverNodeVo;
        }).collect(Collectors.toList());
        return collect;
    }

    /**
     * 查询点评审核节点表数据
     *
     * @return
     */
    public List<AlbumApproverNodeVo> approveFlowsAlbumYelpApprovalNode() {
        List<AlbumYelpApprovalNode> albumYelpApprovalNodeList = albumYelpApprovalNodeService.findAll(null);
        List<AlbumApproverNodeVo> collect = albumYelpApprovalNodeList.parallelStream().map(e -> {
            AlbumApproverNodeVo albumApproverNodeVo = new AlbumApproverNodeVo();
            BeanUtils.copyProperties(e, albumApproverNodeVo);
            return albumApproverNodeVo;
        }).collect(Collectors.toList());
        return collect;
    }

    /**
     * PC端根据知识专辑ID获取知识专辑详情
     *
     * @param pkId
     * @return
     */
    public AlbumInfoDetailPcVo info(String pkId) {
        //返回结果
        AlbumInfoDetailPcVo albumInfoDetailPcVo = new AlbumInfoDetailPcVo();
        //查询专辑详情
        AlbumInfo albumInfo = this.get(pkId);
        //专辑详情(封装数据)
        AlbumInfoDetailVo albumInfoDetailVo = detail(pkId);
        BeanUtils.copyProperties(albumInfoDetailVo, albumInfoDetailPcVo);
        albumInfoDetailPcVo.setVersions(new ArrayList<>());
        //专辑分类赋值
        AlbumCategory albumCategory = albumCategoryMapper.selectById(albumInfo.getCategoryId());
        if (albumCategory.getGrade() == 1) {
            albumInfoDetailPcVo.setCategoryLevel1Id(albumCategory.getPkId());
            albumInfoDetailPcVo.setCategoryLevel1(albumCategory.getName());
        } else {
            AlbumCategory albumCategoryParent = albumCategoryMapper.
                    selectOne(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getPkId, albumCategory.getParent()));
            albumInfoDetailPcVo.setCategoryLevel1Id(albumCategoryParent.getPkId());
            albumInfoDetailPcVo.setCategoryLevel1(albumCategoryParent.getName());
            albumInfoDetailPcVo.setCategoryLevel2Id(albumCategory.getPkId());
            albumInfoDetailPcVo.setCategoryLevel2(albumCategory.getName());
        }
        //判断登录人是否是专辑所属人(是 查看审核中的版本(没有审核中的,查看已发布的) 不是 查看已发布的版本)
        String userId = userService.queryLoginUser();
        if (albumInfo.getOwnerId().equals(userId)) {
            //查询版本信息(审核中)
            AlbumVersion albumVersion1 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", pkId).eq("STATE", 1));
            //查询版本信息(驳回)
            AlbumVersion albumVersion4 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", pkId).eq("STATE", 4));
            if (albumVersion1 != null) {
                albumInfoDetailPcVo.setSummary(albumVersion1.getSummary());
                albumInfoDetailPcVo.setVersionId(albumVersion1.getPkId());
                albumInfoDetailPcVo.setVersionState(albumVersion1.getState());
                albumInfoDetailPcVo.setVersionStateName(AlbumVersionEnum.getMessage(albumVersion1.getState()));
            } else if (albumVersion4 != null) {
                albumInfoDetailPcVo.setSummary(albumVersion4.getSummary());
                albumInfoDetailPcVo.setVersionId(albumVersion4.getPkId());
                albumInfoDetailPcVo.setVersionState(albumVersion4.getState());
                albumInfoDetailPcVo.setVersionStateName(AlbumVersionEnum.getMessage(albumVersion4.getState()));
            } else {
                //生效中的专辑
                AlbumVersion albumVersion2 = albumVersionMapper.
                        selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", pkId).eq("STATE", 2));
                albumInfoDetailPcVo.setSummary(albumVersion2.getSummary());
                albumInfoDetailPcVo.setVersionId(albumVersion2.getPkId());
                albumInfoDetailPcVo.setVersionState(albumVersion2.getState());
                albumInfoDetailPcVo.setVersionStateName(AlbumVersionEnum.getMessage(albumVersion2.getState()));
            }
            albumInfoDetailPcVo.setShowStatus(1);
            albumInfoDetailPcVo.setOwnerId(albumInfo.getOwnerId());
            List<String> actionFlag = new ArrayList<>();
            if (albumInfoDetailPcVo.getVersionState() != 1) {
                actionFlag.add("edit");
            }
            albumInfoDetailPcVo.setActionFlag(actionFlag);
        } else {
            //不是专辑所属人,查看已发布的版本信息
            AlbumVersion albumVersion2 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", pkId).eq("STATE", 2));
            if (albumVersion2 == null) {
                albumInfoDetailPcVo.setShowStatus(0);
                return albumInfoDetailPcVo;
            }
            albumInfoDetailPcVo.setSummary(albumVersion2.getSummary());
            albumInfoDetailPcVo.setVersionId(albumVersion2.getPkId());
            albumInfoDetailPcVo.setVersionState(albumVersion2.getState());
            albumInfoDetailPcVo.setVersionStateName(AlbumVersionEnum.getMessage(albumVersion2.getState()));
            albumInfoDetailPcVo.setOwnerId(albumInfo.getOwnerId());
            albumInfoDetailPcVo.setShowStatus(0);
            albumInfoDetailPcVo.setActionFlag(new ArrayList<>());
        }
        //增加点击量
        if (albumInfoDetailPcVo.getVersionState() == 2) {
            addCountViews(userId, albumInfoDetailPcVo.getPkId());
        }
        return albumInfoDetailPcVo;
    }

    /**
     * 转移专辑负责人
     *
     * @param transferOwnerDto
     */
    public void transferOwner(TransferOwnerDto transferOwnerDto) {
        String pkId = transferOwnerDto.getPkId();
        String target = transferOwnerDto.getTarget();
        AlbumInfo albumInfo = new AlbumInfo();
        albumInfo.setPkId(pkId);
        albumInfo.setOwnerId(target);
        albumInfoMapper.updateById(albumInfo);
    }

    /**
     * 关闭专辑&开启专辑
     *
     * @param transferOwnerDto
     */
    public synchronized void close(TransferOwnerDto transferOwnerDto) {
        String pkId = transferOwnerDto.getPkId();
        AlbumInfo albumInfoById = albumInfoMapper.selectById(pkId);
        AlbumInfo albumInfo = new AlbumInfo();
        albumInfo.setPkId(pkId);
        if (albumInfoById.getState() == 1) {
            albumInfo.setState(0);
        } else {
            albumInfo.setState(1);
        }
        albumInfoMapper.updateById(albumInfo);
    }

    /**
     * PC端编辑知识专辑
     *
     * @param albumInfoEditDto
     */
    @Transactional(rollbackFor = Exception.class)
    public String edit(AlbumInfoEditDto albumInfoEditDto) {
        //判断当前登录人是否是专辑负责人
        String userId = userService.queryLoginUser();
        String albumId = albumInfoEditDto.getPkId();
        AlbumInfo albumInfo = albumInfoMapper.selectById(albumId);
        if (userId.equals(albumInfo.getOwnerId())) {
            //修改专辑信息表
            AlbumInfo albumInfoEdit = new AlbumInfo();
            BeanUtils.copyProperties(albumInfoEditDto, albumInfoEdit);
            albumInfoMapper.updateById(albumInfoEdit);
            //如果有驳回版本的,在驳回的版本上进行修改,不新增版本
            AlbumVersion albumVersion4 = albumVersionMapper.selectOne(new LambdaQueryWrapper<AlbumVersion>().
                    eq(AlbumVersion::getAlbumId, albumId).eq(AlbumVersion::getState, 4));
            AlbumVersion albumVersion = new AlbumVersion();
            if (albumVersion4 != null) {
                //修改版本表数据
                albumVersion.setPkId(albumVersion4.getPkId());
                albumVersion.setApplyUserId(userId);
                albumVersion.setSummary(albumInfoEditDto.getSummary());
                albumVersion.setState(1);
                albumVersion.setApprovalState(1);
                albumVersionMapper.updateById(albumVersion);
                //生成目录
                if (CollectionUtils.isNotEmpty(albumInfoEditDto.getCatalogues())) {
                    //清除该版本下原有目录
                    albumCatalogueService.deleteByVersionId(albumVersion4.getPkId());
                    //在该版本下生成新的目录
                    createCatalogue(albumInfoEditDto.getCatalogues(), albumInfo.getPkId(), albumVersion4.getPkId());
                }
                //修改审核记录(删除原有流程,生成新的流程)
                albumApproveService.deleteByVersionId(albumVersion4.getPkId());
                //生成最新版审核流程数据
                createAlbumApprove(albumInfo.getPkId(), albumVersion4.getPkId());
            } else {
                //新增版本表数据
                albumVersion.setPkId(IdGenerator.getIdStr());
                albumVersion.setAlbumId(albumId);
                albumVersion.setApplyUserId(userId);
                albumVersion.setSummary(albumInfoEditDto.getSummary());
                albumVersion.setUpdateBy(userId);
                albumVersion.setState(1);
                albumVersion.setApprovalState(1);
                albumVersionMapper.insert(albumVersion);
                //在该版本下新增目录
                if (CollectionUtils.isNotEmpty(albumInfoEditDto.getCatalogues())) {
                    //在该版本下生成新的目录
                    createCatalogue(albumInfoEditDto.getCatalogues(), albumInfo.getPkId(), albumVersion.getPkId());
                }
                //生成最新版审核流程数据
                createAlbumApprove(albumInfo.getPkId(), albumVersion.getPkId());
            }
            return "";
        } else {
            return AlbumConst.EDIT_FAIL;
        }
    }

    /**
     * 生成最新版审核流程数据
     */
    private void createAlbumApprove(String albumId, String versionId) {
        //根据专辑Id查询审核表中批次(每次生成批次+1)
        List<AlbumApprove> albumApproveListBatch = albumApproveService.findAll(new LambdaQueryWrapper<AlbumApprove>().eq(AlbumApprove::getAlbumId, albumId));
        //删除老版本审核流程
        albumApproveService.deleteByVersionId(versionId);
        //拉取专辑审核节点表
        List<AlbumApprovalNode> albumApprovalNodeList = albumApprovalNodeService.
                findAll(new QueryWrapper<AlbumApprovalNode>().orderByAsc("IS_FINAL", "CREATE_TIME_"));
        //将版本推送至专辑审核表中(各个节点都推送)
        AtomicInteger sort = new AtomicInteger(1);
        List<AlbumApprove> albumApproveList = albumApprovalNodeList.stream().map(albumApprovalNode -> {
            AlbumApprove albumApprove = new AlbumApprove();
            albumApprove.setPkId(IdGenerator.getIdStr());
            albumApprove.setAlbumId(albumId);
            albumApprove.setVersionId(versionId);
            albumApprove.setApproverId(albumApprovalNode.getApproverId());
            albumApprove.setApproverName(albumApprovalNode.getApproverName());
            albumApprove.setSort(sort.getAndIncrement());
            albumApprove.setIsFinal(albumApprovalNode.getIsFinal());
            albumApprove.setUpdateBy(userService.queryLoginUser());
            albumApprove.setName(albumApprovalNode.getName());
            if (CollectionUtils.isNotEmpty(albumApproveListBatch)) {
                albumApprove.setApprovalBatch(albumApproveListBatch.get(0).getApprovalBatch() + 1);
            }
            return albumApprove;
        }).collect(Collectors.toList());
        //批量添加
        albumApproveService.saveBatch(albumApproveList);
    }

    /**
     * 生成专辑目录
     *
     * @param catalogueDtoList
     * @param albumId
     * @param versionId
     */
    private void createCatalogue(List<AlbumCatalogueDto> catalogueDtoList, String albumId, String versionId) {
        // 向专辑目录表中添加记录
        AtomicInteger i = new AtomicInteger(1);
        catalogueDtoList.forEach(catalogueDto -> {
            AlbumCatalogue albumCatalogue = new AlbumCatalogue();
            albumCatalogue.setPkId(IdGenerator.getIdStr());
            albumCatalogue.setAlbumId(albumId);
            albumCatalogue.setVersionId(versionId);
            albumCatalogue.setName(catalogueDto.getName());
            albumCatalogue.setSort(i.getAndIncrement());
            albumCatalogue.setUpdateBy(userService.queryLoginUser());
            //向专辑目录知识表中添加记录
            if (CollectionUtils.isNotEmpty(catalogueDto.getContents())) {
                List<String> contentIdList = catalogueDto.getContents();
                contentIdList.forEach(contentId -> {
                    AlbumCatalogueKnowledge albumCatalogueKnowledge = new AlbumCatalogueKnowledge();
                    albumCatalogueKnowledge.setPkId(IdGenerator.getIdStr());
                    albumCatalogueKnowledge.setCatalogueId(albumCatalogue.getPkId());
                    albumCatalogueKnowledge.setKnowledgeId(contentId);
                    albumCatalogueKnowledge.setUpdateBy(userService.queryLoginUser());
                    albumCatalogueKnowledgeMapper.insert(albumCatalogueKnowledge);
                });
            }
            albumCatalogueMapper.insert(albumCatalogue);
        });
    }

    /**
     * 删除知识专辑
     *
     * @param ids
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String del(String ids) {
        List<AlbumVersion> albumVersions = albumVersionMapper.selectList(new LambdaQueryWrapper<AlbumVersion>().eq(AlbumVersion::getAlbumId, ids));
        //删除专辑信息
        albumInfoMapper.deleteByIds(ids);
        //删除专辑版本
        albumVersionMapper.deleteByAlbumId(ids);
        albumVersions.forEach(e -> {
            //删除专辑目录及下面的知识
            albumCatalogueService.deleteByVersionId(e.getPkId());
            //删除专辑审核流程
            albumApproveService.deleteByVersionId(e.getPkId());
        });
        //删除专辑的点评信息
        albumYelpService.deleteByAlbumId(ids);
        //删除专辑的点评审核流程
        albumYelpApproveService.deleteByAlbumId(ids);
        return AlbumConst.DEL_SUCCESS;
    }

    /**
     * 移除目录知识和删除点评和回复
     *
     * @param removeKnowledgeAndYelpDto
     */
    public void removeKnowledgeAndYelp(RemoveKnowledgeAndYelpDto removeKnowledgeAndYelpDto) {
        //删除点评记录(如果是一级点评，则对应的回复一起删除)
        if (CollectionUtils.isNotEmpty(removeKnowledgeAndYelpDto.getYelps())) {
            List<String> yelps = removeKnowledgeAndYelpDto.getYelps();
            yelps.stream().forEach(yelp -> {
                //一级点评
                AlbumYelp albumYelp = albumYelpService.get(yelp);
                if (albumYelp.getType() == 1) {
                    //查询下级回复记录
                    List<AlbumYelp> albumYelpList = albumYelpService.findAll(new LambdaQueryWrapper<AlbumYelp>().
                            eq(AlbumYelp::getYelpId, yelp));
                    //收集Id
                    List<String> idList = albumYelpList.parallelStream().map(AlbumYelp::getPkId).collect(Collectors.toList());
                    //清除下级回复
                    albumYelpService.deleteByYelpId(yelp);
                    //清除自己
                    albumYelpService.deleteById(yelp);
                    //清除审批记录表
                    idList.add(yelp);
                    albumYelpApproveService.deleteByYelpId(idList);
                } else {
                    //清除自己
                    albumYelpService.deleteById(yelp);
                    //清除审批记录表
                    List<String> ids = new ArrayList<>();
                    ids.add(yelp);
                    albumYelpApproveService.deleteByYelpId(ids);
                }
            });
        }
        //清除目录表中知识
        Map<String, List<String>> catalogueIdAndKnowledgeList = removeKnowledgeAndYelpDto.getCatalogueIdAndKnowledgeList();
        if (catalogueIdAndKnowledgeList != null) {
            catalogueIdAndKnowledgeList.forEach((k, v) ->
                    albumCatalogueKnowledgeMapper.deleteByCatalogueIds(k, v)
            );
        }
    }


    /**
     * 向统计表中增加数据(相同数据不增加)
     *
     * @param userId  用户Id
     * @param albumId 专辑Id
     */
    private void addCountViews(String userId, String albumId) {
        QueryWrapper<ActionLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("USER_ID", userId);
        queryWrapper.eq("KNOWLEDGE_ID", albumId);
        List<ActionLog> actionLogList = actionLogMapper.selectList(queryWrapper);
        if (CollectionUtils.isEmpty(actionLogList)) {
            ActionLog actionLog = new ActionLog();
            actionLog.setType(2);
            actionLog.setKnowledgeId(albumId);
            actionLog.setUserId(userId);
            actionLogMapper.insert(actionLog);
        }
    }
}
