package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.knowledge.album.entity.dao.AlbumInfo;
import com.redxun.knowledge.album.mapper.AlbumInfoMapper;
import com.redxun.knowledge.entity.dao.ActionLog;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.vo.ActionLogVo;
import com.redxun.knowledge.mapper.ActionLogMapper;
import com.redxun.knowledge.mapper.KnowledgeMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 文件名: ActionLogServiceImpl
 * 作者: Zizg
 * 时间: 2023/1/10
 * 描述:
 * 修改人:
 * 修改时间:
 * 修改内容:
 */
@Service
@Slf4j
public class ActionLogServiceImpl extends SuperServiceImpl<ActionLogMapper, ActionLog> implements BaseService<ActionLog> {

    @Autowired
    private ActionLogMapper actionLogMapper;

    @Autowired
    private KnowledgeMapper knowledgeMapper;

    @Autowired
    private AlbumInfoMapper albumInfoMapper;

    @Override
    public BaseDao<ActionLog> getRepository() {
        return actionLogMapper;
    }

    /**
     * 定时执行方法(每天0时统计知识浏览量)
     * @return
     */
    public boolean countViews() {
        List<ActionLogVo> actionLogVoList = actionLogMapper.countViews(1);
        actionLogVoList.forEach(actionLogVo -> {
            Knowledge knowledge = knowledgeMapper.selectById(actionLogVo.getKnowledgeId());
            // 知识没有被删除且知识状态为5
            if (knowledge != null && knowledge.getApprovalStatus() == 5){
                knowledge.setPkId(actionLogVo.getKnowledgeId());
                // 在原有点击量上增加
                Integer views = knowledge.getViews();
                knowledge.setViews(views + actionLogVo.getSum());
                knowledgeMapper.updateById(knowledge);
            }
        });
        return true;
    }

    /**
     * 定时执行方法(每天0时统计专辑浏览量)
     * @return
     */
    public boolean countAlbumViews() {
        List<ActionLogVo> actionLogVoList = actionLogMapper.countViews(2);
        actionLogVoList.forEach(actionLogVo -> {
            AlbumInfo albumInfo = albumInfoMapper.selectById(actionLogVo.getKnowledgeId());
            // 专辑没有被删除
            if (albumInfo != null){
                albumInfo.setPkId(actionLogVo.getKnowledgeId());
                // 在原有点击量上增加
                Integer views = albumInfo.getViews();
                albumInfo.setViews(views + actionLogVo.getSum());
                albumInfoMapper.updateById(albumInfo);
            }
        });
        return true;
    }

    /**
     * 定时执行方法(统计知识浏览量后清除记录)
     * @return
     */
    public void deleteCountViews() {
        actionLogMapper.deleteCountViews(1);
    }

    /**
     * 定时执行方法(统计知识浏览量后清除记录)
     * @return
     */
    public void deleteCountAlbumViews() {
        actionLogMapper.deleteCountViews(2);
    }
}
